<?php

namespace Drupal\config_normalizer\Config;

use Drupal\config_filter\Config\ReadOnlyStorage;
use Drupal\config_normalizer\ConfigItemNormalizer;
use Drupal\config_normalizer\Plugin\ConfigNormalizerManager;
use Drupal\Core\Config\StorageInterface;

/**
 * Defines the normalizable read only storage.
 *
 * This is like any other StorageInterface, except it does not allow writing
 * and normalizes data on read.
 */
class NormalizedReadOnlyStorage extends ReadOnlyStorage implements StorageInterface {

  /**
   * The config item normalizer.
   *
   * @var \Drupal\config_normalizer\ConfigItemNormalizer
   */
  protected $configItemNormalizer;

  /**
   * Create a NormalizedReadOnlyStorage decorating another storage.
   *
   * @param \Drupal\Core\Config\StorageInterface $storage
   *   The decorated storage.
   * @param \Drupal\config_normalizer\Plugin\ConfigNormalizerManager $normalizer_manager
   *   The normalization manager.
   */
  public function __construct(StorageInterface $storage, ConfigNormalizerManager $normalizer_manager) {
    parent::__construct($storage);
    $this->configItemNormalizer = new ConfigItemNormalizer($normalizer_manager);
  }

  /**
   * {@inheritdoc}
   */
  public function read($name) {
    $data = $this->storage->read($name);

    $data = $this->normalize($name, $data);

    return $data;
  }

  /**
   * {@inheritdoc}
   */
  public function readMultiple(array $names) {
    $list = $this->storage->readMultiple($names);

    foreach ($list as $name => &$data) {
      $data = $this->normalize($name, $data);
    }

    return $list;
  }

  /**
   * Normalizes configuration data.
   *
   * @param string $name
   *   The name of a configuration object to load.
   * @param array $data
   *   The configuration data to normalize.
   *
   * @return array|bool
   *   The configuration data stored for the configuration object name. If no
   *   configuration data exists for the given name, FALSE is returned.
   */
  protected function normalize($name, $data) {
    if (!is_bool($data)) {
      $data = $this->configItemNormalizer->normalize($name, $data);
    }

    return $data;
  }

}
